@@ -1,5 +1,5 @@
 /* Get the contents of an URL.
-   Copyright (C) 2001-2003, 2005-2010, 2012, 2015 Free Software
+   Copyright (C) 2001-2003, 2005-2010, 2012, 2017-2020 Free Software
    Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
@@ -14,7 +14,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 
 #ifdef HAVE_CONFIG_H
@@ -100,10 +100,8 @@ main (int argc, char *argv[])
   set_program_name (argv[0]);
   error_print_progname = maybe_print_progname;
 
-#ifdef HAVE_SETLOCALE
   /* Set locale via LC_ALL.  */
   setlocale (LC_ALL, "");
-#endif
 
   /* Set the text message domain.  */
   bindtextdomain (PACKAGE, relocate (LOCALEDIR));
@@ -142,11 +140,11 @@ main (int argc, char *argv[])
       printf ("%s (GNU %s) %s\n", basename (program_name), PACKAGE, VERSION);
       /* xgettext: no-wrap */
       printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
-License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
+License GPLv3+: GNU GPL version 3 or later <%s>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n\
 "),
-              "2001-2003, 2005-2009");
+              "2001-2020", "https://gnu.org/licenses/gpl.html");
       printf (_("Written by %s.\n"), proper_name ("Bruno Haible"));
       exit (EXIT_SUCCESS);
     }
@@ -193,12 +191,16 @@ Informative output:\n"));
       printf (_("\
   -q, --quiet, --silent       suppress progress indicators\n"));
       printf ("\n");
-      /* TRANSLATORS: The placeholder indicates the bug-reporting address
-         for this package.  Please add _another line_ saying
+      /* TRANSLATORS: The first placeholder is the web address of the Savannah
+         project of this package.  The second placeholder is the bug-reporting
+         email address for this package.  Please add _another line_ saying
          "Report translation bugs to <...>\n" with the address for translation
          bugs (typically your translation team's web or email address).  */
-      fputs (_("Report bugs to <bug-gnu-gettext@gnu.org>.\n"),
-             stdout);
+      printf(_("\
+Report bugs in the bug tracker at <%s>\n\
+or by email to <%s>.\n"),
+             "https://savannah.gnu.org/projects/gettext",
+             "bug-gettext@gnu.org");
     }
 
   exit (status);
@@ -239,6 +241,8 @@ cat_file (const char *src_filename)
     error (EXIT_FAILURE, errno, _("error after reading \"%s\""), src_filename);
 }
 
+#if USEJAVA
+
 /* Exit code of the Java program.  */
 static int java_exitcode;
 
@@ -256,6 +260,8 @@ execute_it (const char *progname,
   return !(java_exitcode == 0 || java_exitcode == 2);
 }
 
+#endif
+
 /* Fetch the URL.  Upon error, use the FILE as fallback.  */
 static void
 fetch (const char *url, const char *file)
@@ -270,20 +276,9 @@ fetch (const char *url, const char *file)
   /* First try: using Java.  */
   {
     const char *class_name = "gnu.gettext.GetURL";
-    const char *gettextjexedir;
     const char *gettextjar;
     const char *args[2];
 
-# if USEJEXE
-    /* Make it possible to override the executable's location.  This is
-       necessary for running the testsuite before "make install".  */
-    gettextjexedir = getenv ("GETTEXTJEXEDIR");
-    if (gettextjexedir == NULL || gettextjexedir[0] == '\0')
-      gettextjexedir = relocate (GETTEXTJEXEDIR);
-# else
-    gettextjexedir = NULL;
-# endif
-
     /* Make it possible to override the gettext.jar location.  This is
        necessary for running the testsuite before "make install".  */
     gettextjar = getenv ("GETTEXTJAR");
@@ -296,7 +291,7 @@ fetch (const char *url, const char *file)
 
     /* Fetch the URL's contents.  */
     java_exitcode = 127;
-    if (!execute_java_class (class_name, &gettextjar, 1, true, gettextjexedir,
+    if (!execute_java_class (class_name, &gettextjar, 1, true, NULL,
                              args,
                              false, true,
                              execute_it, NULL))
